#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

from PyQt4 import QtCore, QtGui

## Hebra que se encarga del proceso de transformar a PDF
class LilypondProcess(QtCore.QThread):
    ## Constructor que inicializa y deja listo el proceso
    #  @param parent Instancia de textEditor que se compilará
    #  @param logWidget Instancia de QTextEdit donde se dejará el registro de salida
    def __init__(self, parent, logWidget):
        super(LilypondProcess, self).__init__(parent)

        ## Información del archivo que se va a compilar
        self.file = QtCore.QFileInfo(parent.curFile)

        ## El nombre del comando para generar el pdf. Algún día será configurable
        self.command = "lilypond"
        ## Un QTextEdit con el que se mostrarán los resultados de la compilación
        #  del archivo
        self.log = logWidget
        self.log.setReadOnly( True )

    ## El corazón del proceso. Se encarga de compilar y guardar la salida
    def run(self):
        ## Proceso compilador
        self.compiler = QtCore.QProcess( )

        self.compiler.setWorkingDirectory( self.file.absolutePath() )
        self.compiler.setReadChannelMode(QtCore.QProcess.MergedChannels)
        self.connect(self.compiler, QtCore.SIGNAL("readyReadStandardOutput()"),
                     self.update)

        ## Archivo a ser compilado en la forma de un QStringList
        self.archive = QtCore.QStringList( self.file.fileName()  )

        print "Creando partitura desde archivo ", self.archive.first()
        self.compiler.start( self.command, self.archive )
        self.compiler.waitForFinished()

        print "Exit Code: ", self.compiler.exitCode()

        self.saveLog()

    ## Actualiza el registro de mensajes
    def update(self):
        self.log.insertPlainText( QtCore.QString( self.compiler.readAll() ))

    ## Guarda el registro de mensajes
    def saveLog(self):
        filename = self.file.absolutePath() + "/" + self.file.completeBaseName() + ".log"

        file = QtCore.QFile(filename)
        if not file.open(QtCore.QFile.WriteOnly | QtCore.QFile.Text):#revisa que pueda guardarse
            QtGui.QMessageBox.critical(self, 'Guardar Registro',
                                       'No se puede guardar el registro')
            return False

        outStream = QtCore.QTextStream(file)
        QtGui.QApplication.setOverrideCursor(QtCore.Qt.WaitCursor) #cursor de espera
        outStream << self.log.toPlainText()
        QtGui.QApplication.restoreOverrideCursor()
                

## Clase que parte el lector de PDF's para ver la parte generada
#  No detecta errores
class ReaderProcess(QtCore.QThread):
    ## Inicializa algunos detalles
    def __init__(self, parent = None):
        super(ReaderProcess, self).__init__(parent)

        lyFile = QtCore.QFileInfo( parent.curFile )
        ## Nombre del archivo PDF a visualizar (obtenido del nombre por defecto)
        self.pdfFile = lyFile.absolutePath() + "/" + lyFile.completeBaseName() + ".pdf"
        ## Comando para leer el PDF
        self.command = "evince"

    ## Parte el visor de pdfs para que sea independiente
    def run(self):
        print "Abriendo archivo ", self.pdfFile
        QtCore.QProcess.startDetached(self.command , QtCore.QStringList(self.pdfFile) )
        return
    
